import sys

from metadocker.common.errors import MetadockerError
from metadocker.db.db import settings


class Argument:
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def call(self, parser):
        parser.add_argument(*self.args, **self.kwargs)


class Command:
    NAME = None
    HELP = None
    ALIASES = []
    ARGUMENTS = []
    CATCH_EXCEPTION = True
    CONFIG_SET = True
    def __init__(self, data):
        self._data = data
        data = data.__dict__

        for k, v in data.items():
            setattr(self, k, v)

    def check(self, raise_exception=False):
        try:
            if self.CONFIG_SET:
                settings.validate()

        except MetadockerError as err:
            print("Erreur de configuration interne à métadocker", file=sys.stderr)
            print(err.get_error(), file=sys.stderr)
            if raise_exception:
                raise err
            return False
        return True


    def start(self):
        if self.check():
            try:
                self.run()
            except MetadockerError as err:
                print("Erreur", file=sys.stderr)
                print(err.get_error(), file=sys.stderr)

    def run(self):
        raise NotImplementedError()


class SpecificCommand(Command):


    def start(self, env):
        env.debug = self.debug
        if self.check():
            try:
                env.read()
                self.run(env)
            except MetadockerError as err:
                print("Erreur", file=sys.stderr)
                print(err.get_error(), file=sys.stderr)



